@@ -1,9 +1,17 @@
-'''Python implementation of PLANET algorithm.'''
+'''Python implementation of PLANET algorithm.
+
+Notes
+-----
+Halir ellipse performs much faster! Somewhat different results though
+between fast guaranteed and Halir -- it would be interesting to do an
+analysis of the difference between the two methods and truth.
+'''
 
 import numpy as np
-from ellipsinator import fit_ellipse_halir
+from ellipsinator import fast_guaranteed_ellipse_estimate as ellipse_fit
 
-def planet(I, alpha, TR, T1_guess=None, mask=None, pc_axis=-1, ret_all=False):
+
+def planet(I, alpha, TR, T1_guess=None, pcs=None, mask=None, pc_axis=-1, ret_all=False):
     '''Simultaneous T1, T2 mapping using phase‐cycled bSSFP.
 
     Parameters
@@ -19,6 +27,9 @@ def planet(I, alpha, TR, T1_guess=None, mask=None, pc_axis=-1, ret_all=False):
     T1_guess : float, optional
         Estimate of expected T1 value (in sec). If None, 1 sec
         is used as the default.
+    pcs : array_like, optional
+        The RF phase-cycle increments used.  If `None`,
+        evenly spaced phase-cycles are assumed on [0, 2pi).
     mask : array_like or None, optional
         Which pixels of I to reconstruct.  If None, the mask
         reconstructs each pixel that has nonzero PC data.
@@ -35,7 +46,8 @@ def planet(I, alpha, TR, T1_guess=None, mask=None, pc_axis=-1, ret_all=False):
         Estimate of T1 values (in sec).
     T2 : array_like
         Estimate of T2 values (in sec).
-
+    df : array_like
+        Estimate of off-resonance (in Hz).
     phi : array_like, optional
     Xc : array_like, optional
     Yc : array_like, optional
@@ -58,26 +70,30 @@ def planet(I, alpha, TR, T1_guess=None, mask=None, pc_axis=-1, ret_all=False):
            phase‐cycled balanced steady‐state free precession."
            Magnetic resonance in medicine 79.2 (2018): 711-722.
     '''
-   # Phase cycles to the back
+    # Phase cycles to the back
     I = np.moveaxis(I, pc_axis, -1)
     npcs = I.shape[-1]
     sh = I.shape[:-1]
 
+    if pcs is None:
+        pcs = np.linspace(0, 2*np.pi, npcs, endpoint=False)
+    else:
+        assert len(pcs) == npcs, "pcs and data must match!"
+
     # alpha can either be a scalar or array
     # Choose an intial estimate for T1
     if T1_guess is None:
-        T1_guess = 1 # 1sec arbitrariliy
+        T1_guess = 1  # 1sec arbitrariliy
 
     # Fit all ellipses that are nonzero
     if mask is None:
-        recon_idx = np.nonzero(np.sum(np.abs(I).reshape((-1, npcs)), axis=-1))[0]
+        recon_idx = np.nonzero(
+            np.sum(np.abs(I).reshape((-1, npcs)), axis=-1))[0]
     else:
         recon_idx = np.nonzero(mask.flatten())[0]
-    #recon_idx = list(range(np.prod(sh)))
-    ellipse_coefs = fit_ellipse_halir(np.take(
-        I.reshape((-1, npcs)),
-        recon_idx,
-        axis=0))
+
+    I = np.take(I.reshape((-1, npcs)), recon_idx, axis=0)
+    ellipse_coefs = ellipse_fit(I)
 
     # Filter out all the fitting failures
     failure_idx = np.where(np.sum(np.abs(ellipse_coefs), axis=-1) == 0)[0]
@@ -156,37 +172,62 @@ def planet(I, alpha, TR, T1_guess=None, mask=None, pc_axis=-1, ret_all=False):
 
     T1 = np.zeros(np.prod(sh))
     calpha = np.cos(alpha)
-    T1[recon_idx] = -TR/np.log(((a*(1 + calpha - ab*calpha) - b)/(a*(1 + calpha - ab) - b*calpha)))
+    T1[recon_idx] = -TR/np.log(
+        ((a*(1 + calpha - ab*calpha) - b)/(a*(1 + calpha - ab) - b*calpha)))
+
+    # compute off-resonance estimates
+    Xn = I.real*c[..., None] + I.imag*s[..., None]
+    Yn = I.imag*c[..., None] - I.real*s[..., None]
+    Coef = (a - b)/(a*np.sqrt(1 - bb2))  # Coef=A/B
+    TanT = Coef[..., None]*(Yn - Yc[..., None])/(Xn - Xc[..., None])
+    T = np.arctan(TanT)  # defined on [-pi/2, pi/2]
+    # unwrapping of T(k) to [-pi,pi]
+    idx0 = Xn < Xc[..., None]
+    idx_greater = np.logical_and(idx0, (Yn - Yc[..., None]) >= 0)
+    # T[idx_greater] = np.pi - T
+    np.putmask(T, idx_greater, np.pi - T[idx_greater])
+    idx_lesser = np.logical_and(idx0, (Yn - Yc[..., None]) < 0)
+    # T[idx_lesser] = -1*np.pi + T
+    np.putmask(T, idx_lesser, -1*np.pi + T[idx_lesser])
+    CosT = np.cos(T)
+    c = (CosT - b[..., None])/(b[..., None]*CosT - 1)
+    A = np.vstack((np.cos(pcs), np.sin(pcs))).T
+    x = np.linalg.lstsq(A, c.T, rcond=None)[0]
+    df = np.zeros(T1.shape)
+    df[recon_idx] = -1*np.arctan2(x[1, :], x[0, :])/(2*np.pi*TR)
 
     if ret_all:
-       # Pack the rest of the result matrices
-       Phimap = np.zeros(np.prod(sh))
-       Phimap[recon_idx] = phi
-       Xcmap = np.zeros(np.prod(sh))
-       Xcmap[recon_idx] = Xc
-       Ycmap = np.zeros(np.prod(sh))
-       Ycmap[recon_idx] = Yc
-       Amap = np.zeros(np.prod(sh))
-       Amap[recon_idx] = a
-       Bmap = np.zeros(np.prod(sh))
-       Bmap[recon_idx] = b
+        # pack result matrices
+        Phimap = np.zeros(np.prod(sh))
+        Phimap[recon_idx] = phi
+        Xcmap = np.zeros(np.prod(sh))
+        Xcmap[recon_idx] = Xc
+        Ycmap = np.zeros(np.prod(sh))
+        Ycmap[recon_idx] = Yc
+        Amap = np.zeros(np.prod(sh))
+        Amap[recon_idx] = a
+        Bmap = np.zeros(np.prod(sh))
+        Bmap[recon_idx] = b
+
+        return(
+            np.reshape(Mmap, sh),
+            np.reshape(T1, sh),
+            np.reshape(T2, sh),
+            np.reshape(df, sh),
+            Phimap,
+            Xcmap,
+            Ycmap,
+            Amap,
+            Bmap,
+        )
+
+    return (
+        np.reshape(Mmap, sh),
+        np.reshape(T1, sh),
+        np.reshape(T2, sh),
+        np.reshape(df, sh),
+    )
 
-       return(
-          np.reshape(Mmap, sh),
-          np.reshape(T1, sh),
-          np.reshape(T2, sh),
-          np.reshape(Phimap, sh),
-          np.reshape(Amap, sh),
-          np.reshape(Bmap, sh),
-          np.reshape(Xcmap, sh),
-          np.reshape(Ycmap, sh),
-       )
-    else:
-       return(
-          np.reshape(Mmap, sh),
-          np.reshape(T1, sh),
-          np.reshape(T2, sh),
-       )
 
 if __name__ == '__main__':
-   pass
+    pass
